/******************************************************************************
Sauna - Monitor Version 5.0.1
 
Software für den Arduino Uno in Verbindung mit folgender Zusatz-Hardware:
- ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler
- 12864 Graphic LCD Display mit seriellem Anschluss (RSPI)
- Temperatur-/Luftfeuchtesensor DHT22
- 1 Wire Temperatursensoren DS18x20
- ELV MiniWavePlayer MWP1
 
(c) 2014 by Jens-P. Stern, http://hausautomation.stern-av.de
Lizenziert unter einer Creative Commons Namensnennung – Weitergabe unter gleichen Bedingungen 4.0 International Lizenz.
Ggf. bestehende Lizenzrechte Dritter bleiben hiervon unberührt.
Die Software wird angeboten wie sie ist und ohne Garantien jedweder Art. Der Nutzer übernimmt somit das vollständige Risiko deren Gebrauchs.
Unter keinen Umständen ist der Verfasser für irgendwelche Schäden haftbar, die aus dem Gebrauch, dem Missbrauch oder dem Unvermögen, die Software anzuwenden, entstehen.
Schadenersatz hierfür ist ausgeschlossen.
******************************************************************************/
 
/******************************************************************************
Übersicht der Pinbelegung des Arduino Uno:
 
PIN  0 (Serial RX)               - intern belegt
PIN  1 (Serial TX)               - intern belegt
PIN  2 (External Interrupt)      - 1Wire-BUS
PIN  3 (External Interrupt, PWM) - ELV MiniWavePlayer MWP1 - Audiokanal 07 (Es sind fünfzehn Miunten vergangen)
PIN  4                           - ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler - Taster 1 (Start)
PIN  5 (PWM)                     - ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler - Taster 2 (Stopp)
PIN  6 (PWM)                     - ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler - Taster 3 (Reset)
PIN  7                           - ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler - Taster 4 (Musik an/aus)
PIN  8                           - ELV MiniWavePlayer MWP1 - Audiokanal 01 (Entspannungsmusik)
PIN  9 (PWM)                     - ELV MiniWavePlayer MWP1 - Audiokanal 02 (Der Saunagang wurde gestartet, viel Spaß)
PIN 10 (PWM)                     - ELV MiniWavePlayer MWP1 - Audiokanal 03 (Der Saunagang wurde gestoppt)
PIN 11 (PWM)                     - ELV MiniWavePlayer MWP1 - Audiokanal 04 (Der Saunatimer wurde zurück gesetzt)
PIN 12                           - ELV MiniWavePlayer MWP1 - Audiokanal 05 (Es sind fünf Miunten vergangen)
PIN 13 (LED)                     - ELV MiniWavePlayer MWP1 - Audiokanal 06 (Es sind zehn Miunten vergangen)
PIN 14 (A0)                      - Temperatur-/Luftfeuchtesensor DHT22 (als Digitalpin definiert)
PIN 15 (A1)                      - 12864 Graphic LCD Display mit seriellem Anschluss (RSPI) - clockPin --> SCK(EN) (als Digitalpin definiert)
PIN 16 (A2)                      - 12864 Graphic LCD Display mit seriellem Anschluss (RSPI) - latchPin --> CS(RS) (als Digitalpin definiert)
PIN 17 (A3)                      - 12864 Graphic LCD Display mit seriellem Anschluss (RSPI) - dataPin --> SID(RW) (als Digitalpin definiert)
PIN 18 (A4)                      - ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler - I2C-BUS - SDA
PIN 19 (A5)                      - ELV I2C-4-Digit LED Display I2C-4DLED mit Temperaturfühler - I2C-BUS - SCL
******************************************************************************/
 
/******************************************************************************
Bibliotheken / globale Konstanten / Hardware Initialisierung
******************************************************************************/
#include "Wire.h" // I2C Bibliothek einbeziehen
#include "I2C_4DLED.h" // I2C_4DLED Bibliothek einbeziehen
#include "MCP9801.h" // I2C_4DLED Temperatursensor Bibliothek einbeziehen
#include "LCD12864RSPI.h" // LCD Bibliothek einbeziehen
#include "DFrobot_bmp.h" // LCD Bibliothek einbeziehen (DFrobot Logo)
#include "DFrobot_char.h" // LCD Bibliothek einbeziehen (Zeichensatz)
#include "stdlib.h" // LCD Bibliothek einbeziehen
#include "DHT.h" // DHT22 Bibliothek einbeziehen
#include <OneWire.h> // 1Wire Bibliothek einbeziehen
#include <DallasTemperature.h> // DS18x20 Bibliothek einbeziehen
 
#define AR_SIZE( a ) sizeof( a ) / sizeof( a[0] ) // 12864 LCD Größe definieren
 
#define DHTPIN 14     // PIN für den DHT22 definieren
#define DHTTYPE DHT22   // Typ des DHT-Moduls festlegen
DHT dht(DHTPIN, DHTTYPE); // DTH Modul initialisieren
 
#define ONE_WIRE_BUS 2 // PIN für den 1Wire-Bus (18B20) definieren
#define TEMPERATURE_PRECISION 9 // Messgenauigkeit für den 18B20 definieren
OneWire oneWire(ONE_WIRE_BUS); // Setup einer 1Wire Instanz zur Kommunikation mit allen 1Wire Geräten (nicht nur dieses Herstellers)
DallasTemperature sensors(&oneWire); // Die 1Wire Referenz auf "Dallas Temperature" festlegen
 
byte saa1064 = 0x70 >> 1; // I2C Bus Adresse für SAA1064 definieren -> SAA1064
const uint8_t i2cAddressSAA1064 = 0x70; // I2C Bus Adresse für SAA1064 definieren -> ELV I2C_4DLED
const uint8_t i2cAddressMCP9801 = 0x90; // I2C Bus Adresse für MCP9801 definieren -> ELV I2C_4DLED
 
const uint32_t crystalFrequency     = 16000000UL; // Frequenz des Oszillators in Hz -> 16 MHz
const uint8_t timerFrequency        = 1UL; // Frequenz des Timer 1 in Hz -> 10 Hz
 
const uint8_t pinStartButton        = 4; // PIN für die Taste START definieren
const uint8_t pinStopButton         = 5; // PIN für die Taste STOP definieren
const uint8_t pinResetButton        = 6; // PIN für die Taste RESET definieren
const uint8_t pinMusikButton        = 7; // PIN für die Taste MUSIK definieren
 
const int audiokanal01    = 8;  // Pinbelegung des Audio-Players (Entspannungsmusik)
const int audiokanal02    = 9;  // Pinbelegung des Audio-Players (Der Saunagang wurde gestartet, viel Spaß)
const int audiokanal03    = 10; // Pinbelegung des Audio-Players (Der Saunagang wurde gestoppt)
const int audiokanal04    = 11; // Pinbelegung des Audio-Players (Der Saunatimer wurde zurück gesetzt)
const int audiokanal05    = 12; // Pinbelegung des Audio-Players (Es sind fünf Miunten vergangen)
const int audiokanal06    = 13; // Pinbelegung des Audio-Players (Es sind zehn Miunten vergangen)
const int audiokanal07    = 3;  // Pinbelegung des Audio-Players (Es sind fünfzehn Miunten vergangen)
 
/******************************************************************************
globale Variablen
******************************************************************************/
uint8_t running = 0; // Indikator, ob Stoppuhr steht (0) oder läuft (1)
uint16_t time = 0; // vergangene Zeit in 0,1 s Schritten
volatile uint8_t timerflag = 0; // Flag zur Anzeige, dass der Timerinterrupt aktiviert wurde
int32_t I2C_temperatur = 0; // Temperaturwert des ELV I2C_4DLED MCP9801
I2C_4DLED::OutputCurrent outputCurrent = I2C_4DLED::OUTPUT_CURRENT_0_MA; // Ausgangsstrom des SAA1064 (Helligkeit)
int schwitzminuten = 0; // Dauer des Saunagangs
int anzeigezaehler = 0; // Zeitliche Festlegung der Anzeigedauer
int anzeigeflag = 0; // Schritte der Anzeigedauer für die unterschiedlichen Darstellungen
int runningindicator = 0; // Saunauhr an/aus
int musikflag = 1; // Musik an (1)/aus (0) (toggeln)
 
/******************************************************************************
Funktionen
******************************************************************************/
void timerInit() // Initialisierung Timer 1
{
  TCCR1A = (0 << WGM11) | (0 << WGM10); // Modus 4: CTC
  TCCR1B = (0 << WGM13) | (1 << WGM12); // Modus 4: CTC
  TCNT1  = 0; // Counter zurücksetzen
  TCCR1B |= (1 << CS12) | (0 << CS11) | (0 << CS10); // Prescaler: 256
  OCR1A = (uint16_t)(crystalFrequency / (timerFrequency * 256UL)) - 1; // Output Compare Register für 10 Hz bei einem Prescaler von 256 einstellen
  TIFR1 = (1 << OCF1A); // Compare Match Flag zurücksetzen
  TIMSK1 |= (1 << OCIE1A); // Compare Match Interrupt aktivieren
}
 
ISR(TIMER1_COMPA_vect) // Interrupt-Routine des Timer 1, welcher im 0,1s-Takt auslöst
{
  timerflag = 1; // Timerflag setzen, so dass die loop-Routine erkennt, dass 0,1 s vergangen sind
}
 
void clearDisplay() // Alle Digits der LED-Anzeige löschen
{
  Wire.beginTransmission(saa1064);
  Wire.write(1); // instruction byte - erstes Digit ist Digit 1
  Wire.write(0); // digit 1 (rechts)
  Wire.write(0); // digit 2
  Wire.write(0); // digit 3
  Wire.write(0); // digit 4 (links)
  Wire.endTransmission();
}
 
/******************************************************************************
Setup
******************************************************************************/
void setup()
{
  // Timer initialisieren
  noInterrupts();
  timerInit();
  interrupts();
 
  Wire.begin(); // I2C-Modul initialisieren
  FourDigitLedDisplay.begin(i2cAddressSAA1064); // 4-Digit-LED-Anzeige mit I2C-Adresse des SAA1064 initialisieren
  FourDigitLedDisplay.setDisplayOutputCurrent(I2C_4DLED::OUTPUT_CURRENT_12_MA); // Helligkeit der 4-Digit-LED-Anzeige einstellen
 
  TemperatureSensor.begin(i2cAddressMCP9801); // Temperatursensor mit I2C-Adresse des MCP9801 initialisieren
  TemperatureSensor.setADCResolution(TemperatureSensor.RESOLUTION_12BIT); // Temperatursensor auf eine Auflösung von 12 Bit umschalten
 
  sensors.begin(); // Start der 1Wire Library
 
  // LCD Display initialisieren und ansteuern
  unsigned char zeile_1_1[]="Sauna - Monitor "; // Text für Zeile 1 (0)
  unsigned char zeile_2_1[]="(c) Jens-P.Stern"; // Text für Zeile 2 (1)
  unsigned char zeile_3_1[]="Initialisierung "; // Text für Zeile 3 (2)
  unsigned char zeile_4_1[]="bitte warten    "; // Text für Zeile 4 (3)
  LCDA.initDriverPin(15,16,17); // Das SPI Interface des LCD Displays initialisieren und die PINs 15, 16 und 17 definieren
  LCDA.Initialise(); // LCD Display initialisieren
  delay(100);
  LCDA.DrawFullScreen(logo); // DFrobot Logo anzeigen
  delay(2000);
  randomSeed(0);
  LCDA.CLEAR(); // LCD Display löschen
  delay(100);
  LCDA.DisplayString(0,0,zeile_1_1,16); // Zeile 1 (0) schreiben
  LCDA.DisplayString(1,0,zeile_2_1,16); // Zeile 2 (1) schreiben
  LCDA.DisplayString(2,0,zeile_3_1,16); // Zeile 3 (2) schreiben
  LCDA.DisplayString(3,0,zeile_4_1,16); // Zeile 4 (3) schreiben
 
  // Pins für den Audio-Player als Ausgänge setzen
  pinMode(audiokanal01, OUTPUT);
  pinMode(audiokanal02, OUTPUT);
  pinMode(audiokanal03, OUTPUT);
  pinMode(audiokanal04, OUTPUT);
  pinMode(audiokanal05, OUTPUT);
  pinMode(audiokanal06, OUTPUT);
  pinMode(audiokanal07, OUTPUT);
 
  // Schaltausgänge auf High setzen (+5V)
  digitalWrite(audiokanal01, HIGH);
  digitalWrite(audiokanal02, HIGH);
  digitalWrite(audiokanal03, HIGH);
  digitalWrite(audiokanal04, HIGH);
  digitalWrite(audiokanal05, HIGH);
  digitalWrite(audiokanal06, HIGH);
  digitalWrite(audiokanal07, HIGH);
 
  // Pins für die Taster als Eingänge setzen
  pinMode(pinStartButton, INPUT);
  pinMode(pinStopButton, INPUT);
  pinMode(pinResetButton, INPUT);
  pinMode(pinMusikButton, INPUT);
 
  // interne Pullup-Widerstände für die Tastereingänge aktivieren
  digitalWrite(pinStartButton, HIGH);
  digitalWrite(pinStopButton, HIGH);
  digitalWrite(pinResetButton, HIGH);
  digitalWrite(pinMusikButton, HIGH);
 
  // Segmenttest der LED Anzeige durchführen
  FourDigitLedDisplay.testDisplaySegments(1);
  delay(2000);
  FourDigitLedDisplay.testDisplaySegments(0);
 
  // LED Anzeige ansteuern
  Wire.beginTransmission(saa1064); // JPS
  Wire.write(1); // instruction byte - erstes Digit ist Digit 1
  Wire.write(B00011111); // J
  Wire.write(B01110011); // P
  Wire.write(B01101101); // S
  Wire.write(B00000000); //
  Wire.endTransmission();
  delay(2000);
  FourDigitLedDisplay.clearDisplay(); // LED Display löschen
 
  digitalWrite(audiokanal04, LOW); // Audio Mitteilung RESET
 
  // Variablen zurücksetzen
  running = 0;
  time = 0;
  schwitzminuten = 0;
  runningindicator = 0;
 
  // LED Anzeige ansteuern
  FourDigitLedDisplay.clearDisplay(); // LED Display löschen
  Wire.beginTransmission(saa1064); // rESt
  Wire.write(1); // instruction byte - erstes Digit ist Digit 1
  Wire.write(B01010000); // digit r
  Wire.write(B01111001); // digit E
  Wire.write(B01101101); // digit S
  Wire.write(B01111000); // digit t
  Wire.endTransmission();
  delay(50);
  digitalWrite(audiokanal04, HIGH);
 
  // LCD Display ansteuern
  unsigned char zeile_1_2[]="Sauna - Monitor "; // Text für Zeile 1 (0)
  unsigned char zeile_2_2[]="(c) Jens-P.Stern"; // Text für Zeile 2 (1)
  unsigned char zeile_3_2[]="Das System ist  "; // Text für Zeile 3 (2)
  unsigned char zeile_4_2[]="betriebsbereit  "; // Text für Zeile 4 (3)
  LCDA.CLEAR(); // LCD Display löschen
  LCDA.DisplayString(0,0,zeile_1_2,16); // Zeile 1 (0) schreiben
  LCDA.DisplayString(1,0,zeile_2_2,16); // Zeile 2 (1) schreiben
  LCDA.DisplayString(2,0,zeile_3_2,16); // Zeile 3 (2) schreiben
  LCDA.DisplayString(3,0,zeile_4_2,16); // Zeile 4 (3) schreiben
  delay(2000);
}
 
/******************************************************************************
Loop
******************************************************************************/
void loop()
{
  // Auslesen des DHT22. Das Auslesen der Temperatur und Feuchte kann bis zu 250 ms dauern. Die Werte können bis zu 2 Sek alt sein (sehr langsamer Sensor)
  float DHT22_temperatur = dht.readTemperature(); // Temperatur des DHT22 (diese Daten werden nicht weiter genutzt, da Temperaturmeßbereich nur bis 80° C)
  float DHT22_feuchte = dht.readHumidity(); // Luftfeuchtigkeit des DHT22 (in der Sauna)
  int DHT22_feuchte_innen_int=(int)(DHT22_feuchte+.5); // Float in Integer wandeln mit kaufmännischer Rundung
  char DHT22_feuchte_innen_char[16]; // String Variable anlegen
  sprintf(DHT22_feuchte_innen_char,"%d", DHT22_feuchte_innen_int); // Integer in String wandeln, Text für Zeile 3 (2)
 
  sensors.requestTemperatures(); // Temperaturen der 1Wire-Sensoren abfragen
 
  float onewire_sauna_aussen_float = sensors.getTempCByIndex(0); // Erster 1Wire-Sensor (außerhalb der Sauna)
  int onewire_sauna_aussen_int=(int)(onewire_sauna_aussen_float+.5); // Float in Integer wandeln mit kaufmännischer Rundung
  char onewire_sauna_aussen_char[16]; // String Variable anlegen
  sprintf(onewire_sauna_aussen_char,"%d", onewire_sauna_aussen_int); // Integer in String wandeln, Text für Zeile 3 (2)
 
  float onewire_sauna_innen_float = sensors.getTempCByIndex(1); // Zweiter 1Wire-Sensor (in der Sauna)
  int onewire_sauna_innen_int=(int)(onewire_sauna_innen_float+.5); // Float in Integer wandeln mit kaufmännischer Rundung
  char onewire_sauna_innen_char[16]; // String Variable anlegen
  sprintf(onewire_sauna_innen_char,"%d", onewire_sauna_innen_int); // Integer in String wandeln, Text für Zeile 3 (2)
 
  // Auslesen des Temperatursensors vom I2C_4DLED in Grad Celsius (der Temperaturwert ist in Feskommadarstellung mit 4 Nachkommastellen gespeichert)
  float I2C_tempanzeige = 1.0; // Temperaturwert des ELV I2C_4DLED MCP9801 (an der LED-Anzeige, diese Daten werden nicht weiter genutzt, können aber später zur Überwachung der Anzeigetemperatur herangezogen werden)
  float I2C_temperatur = TemperatureSensor.readTemperature();
  I2C_tempanzeige = (((I2C_temperatur + 500) / 1000)/10);
  int I2C_sauna_anzeige_int=(int)(I2C_tempanzeige+.5); // Float in Integer wandeln mit kaufmännischer Rundung
  char I2C_sauna_anzeige_char[16]; // String Variable anlegen
  sprintf(I2C_sauna_anzeige_char,"%d", I2C_sauna_anzeige_int); // Integer in String wandeln, Text für Zeile 3 (2)
 
  // Alle 0,1 s die Zeit inkrementieren, die neue Zeit ausgeben und die Taster auswerten
  if(timerflag == 1)
  {
    timerflag = 0;
    anzeigezaehler++;
    if(anzeigezaehler == 1) // Saunagangdauer (schwitzzeit)
    {
      anzeigeflag = 1;
    }
    if(anzeigezaehler == 15) // "t °C"
    {
      anzeigeflag = 2;
    }
    if(anzeigezaehler == 16)  // Temp Sauna
    {
      anzeigeflag = 3;
    }
    if(anzeigezaehler == 20) // F %
    {
      anzeigeflag = 4;
    }
    if(anzeigezaehler == 21) // Feuchte Sauna
    {
      anzeigeflag = 5;
    }
    if(anzeigezaehler == 25) // ZEIT
    {
      anzeigeflag = 0;
      anzeigezaehler = 0;
    }
 
    if(running == 1) // die Zeit nur erhöhen, wenn die Stoppuhr aktiviert ist
    {
      time++;
      runningindicator++;
        if(runningindicator > 2)
        {
          runningindicator = 0;
        }
      schwitzminuten = (time/60);
    }
 
    if (time == 300) // nach 300 Sekunden Audio Ausgabe 5 MINUTEN
       {
       digitalWrite(audiokanal05, LOW);
       delay(50);
       digitalWrite(audiokanal05, HIGH);
       if (musikflag == 1) // Audio Ausgabe ENTPANNUNGSMUSIK, wenn die Musik eingeschaltet ist
       {
       delay(3000);
       digitalWrite(audiokanal01, LOW);
       delay(50);
       digitalWrite(audiokanal01, HIGH);
       }
       }
 
    if (time == 600) // nach 600 Sekunden Audio Ausgabe 10 MINUTEN
       {
       digitalWrite(audiokanal06, LOW);
       delay(50);
       digitalWrite(audiokanal06, HIGH);
       if (musikflag == 1) // Audio Ausgabe ENTPANNUNGSMUSIK, wenn die Musik eingeschaltet ist
       {
       delay(3000);
       digitalWrite(audiokanal01, LOW);
       delay(50);
       digitalWrite(audiokanal01, HIGH);
       }
       }
 
    if (time == 900) // nach 900 Sekunden Audio Ausgabe 15 MINUTEN
       {
       digitalWrite(audiokanal07, LOW);
       delay(50);
       digitalWrite(audiokanal07, HIGH);
       if (musikflag == 1) // Audio Ausgabe ENTPANNUNGSMUSIK, wenn die Musik eingeschaltet ist
       {
       delay(3000);
       digitalWrite(audiokanal01, LOW);
       delay(50);
       digitalWrite(audiokanal01, HIGH);
       }
       }
 
    if(anzeigeflag == 0) // Anzeigeperiode 1
    {
    // LED Anzeige ansteuern
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "ZEIt" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01011011); // digit Z
    Wire.write(B01111001); // digit E
    Wire.write(B00000110); // digit I
    Wire.write(B01111000); // digit t
    Wire.endTransmission();
 
    if(running == 0) // Sanagang läuft nicht
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Raumtemperatur: "; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Grad Celsius    "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)onewire_sauna_aussen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
 
    if(running == 1) // Saunagang läuft
    {
    // LCD Display ansteuern
    char schwitzminuten_char[16]; // String Variable anlegen
    sprintf(schwitzminuten_char,"%d", schwitzminuten); // Integer in String wandeln, Text für Zeile 3 (2)
    unsigned char zeile_1[]="Saunatimer: EIN "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunagangdauer: "; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Minuten         "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)schwitzminuten_char,2); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
    }
 
    if(anzeigeflag == 1) // Anzeigeperiode 2
    {
    FourDigitLedDisplay.writeDecimal(schwitzminuten, 0); // Saunagangdauer auf der LED Anzeige ausgeben
 
    if(running == 0) // Saunagang läuft nicht
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Raumtemperatur: "; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Grad Celsius    "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)onewire_sauna_aussen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    Wire.beginTransmission(saa1064); // Laufanzeige (Striche von unten nach oben) auf der LED Anzeige zurücksetzen
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B00000000); // digit _
    Wire.endTransmission();
    }
 
    if(running == 1) // Saunagang läuft
    {
    // LCD Display ansteuern
    char schwitzminuten_char[16]; // String Variable anlegen
    sprintf(schwitzminuten_char,"%d", schwitzminuten); // Integer in String wandeln, Text für Zeile 3 (2)
    unsigned char zeile_1[]="Saunatimer: EIN "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunagangdauer: "; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Minuten         "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)schwitzminuten_char,2); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    // Laufanzeige (Striche von unten nach oben) auf der LED Anzeige ausgeben
    if (runningindicator == 0)
       {
       Wire.beginTransmission(saa1064);
       Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
       Wire.write(B00001000); // digit _
       Wire.endTransmission();
       }
    if (runningindicator == 1)
       {
       Wire.beginTransmission(saa1064);
       Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
       Wire.write(B01000000); // digit _
       Wire.endTransmission();
       }
    if (runningindicator == 2)
       {
       Wire.beginTransmission(saa1064);
       Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
       Wire.write(B00000001); // digit _
       Wire.endTransmission();
       }
    }
    }
 
    if(anzeigeflag == 2) // Anzeigeperiode 3
    {
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "t °C" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01111000); // digit t
    Wire.write(B00000000); // digit .
    Wire.write(B01100011); // digit °
    Wire.write(B00111001); // digit C
    Wire.endTransmission();
 
    if(running == 0) // Saunagang läuft nicht
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunatemperatur:"; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Grad Celsius    "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)onewire_sauna_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
 
    if(running == 1) // Saunagang läuft
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: EIN"; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunatemperatur:"; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Grad Celsius    "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)onewire_sauna_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
    }
 
    if(anzeigeflag == 3) // Anzeigeperiode 4
    {
    FourDigitLedDisplay.writeDecimal(onewire_sauna_innen_int, 0); // Sauna Innentemperatur auf der LED Anzeige ausgeben
 
    if(running == 0) // Saunagang läuft nicht
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunatemperatur: "; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Grad Celsius    "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)onewire_sauna_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
 
    if(running == 1) // Saunagang läuft
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: EIN "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunatemperatur: "; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Grad Celsius    "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)onewire_sauna_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
    }
 
    if(anzeigeflag == 4) // Anzeigeperiode 5
    {
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "f°/o" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01110001); // digit f
    Wire.write(B01100011); // digit °
    Wire.write(B01010010); // digit /
    Wire.write(B01011100); // digit o
    Wire.endTransmission();
 
    if(running == 0) // Saunagang läuft nicht
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunaluftfeuchte"; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Prozent         "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)DHT22_feuchte_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
 
    if(running == 1) // Saunagang läuft
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: EIN "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunaluftfeuchte"; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Prozent         "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)DHT22_feuchte_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
    }
 
    if(anzeigeflag == 5) // Anzeigeperiode 6
    {
    FourDigitLedDisplay.writeDecimal(DHT22_feuchte_innen_int, 0); // Sauna Innenfeuchte auf der LED Anzeige ausgeben
 
    if(running == 0) // Saunagang läuft nicht
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunaluftfeuchte"; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Prozent         "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)DHT22_feuchte_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
 
    if(running == 1) // Saunagang läuft
    {
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: EIN "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="Saunaluftfeuchte"; // Text für Zeile 2 (1)
    unsigned char zeile_4[]="Prozent         "; // Text für Zeile 4 (3)
    LCDA.CLEAR(); // LCD Display löschen
    LCDA.DisplayString(0,0,zeile_1,16); //Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(2,0,(unsigned char *)DHT22_feuchte_innen_char,3); // Zeile 2 (1) schreiben (die LCDA Funktion erwartet "unsigned char arrays")
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
    }
    }
 
    // Stoppuhr laufen lassen, wenn der Start-Taster gedrückt wurde
    if(digitalRead(pinStartButton) == 0)
    {
    digitalWrite(audiokanal02, LOW); // Audio Ausgabe START
    delay(50);
    digitalWrite(audiokanal02, HIGH);
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "Strt" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01101101); // digit S
    Wire.write(B01111000); // digit t
    Wire.write(B01010000); // digit r
    Wire.write(B01111000); // digit t
    Wire.endTransmission();
 
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: EIN "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="                "; // Text für Zeile 2 (1)
    unsigned char zeile_3[]="Saunagang       "; // Text für Zeile 3 (2)
    unsigned char zeile_4[]="gestartet       "; // Text für Zeile 4 (3)
    LCDA.CLEAR();
    LCDA.DisplayString(0,0,zeile_1,16); // Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 2 (1) schreiben
    LCDA.DisplayString(2,0,zeile_3,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    if (musikflag == 1) // Audio Ausgabe ENTPANNUNGSMUSIK, wenn die Musik eingeschaltet ist
    {
    delay(4000);
    digitalWrite(audiokanal01, LOW);
    }
    delay(50);
    running = 1;
    digitalWrite(audiokanal01, HIGH);
    }
 
    // Stoppuhr anhalten, wenn der Stopp-Taster gedrückt wurde
    if(digitalRead(pinStopButton) == 0)
    {
    digitalWrite(audiokanal03, LOW); // Audio Ausgabe STOPP
    running = 0;
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "StoP" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01101101); // digit S
    Wire.write(B01111000); // digit t
    Wire.write(B01011100); // digit o
    Wire.write(B01110011); // digit P
    Wire.endTransmission();
 
   // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="                "; // Text für Zeile 2 (1)
    unsigned char zeile_3[]="Saunagang       "; // Text für Zeile 3 (2)
    unsigned char zeile_4[]="gestoppt        "; // Text für Zeile 4 (3)
    LCDA.CLEAR();
    LCDA.DisplayString(0,0,zeile_1,16); // Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 2 (1) schreiben
    LCDA.DisplayString(2,0,zeile_3,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    delay(4000);
    digitalWrite(audiokanal03, HIGH);
    }
 
    // Stoppuhr anhalten und zurücksetzen, wenn der Reset-Taster gedrückt wurde oder der Timer bei 1200 (20 Minuten) steht
    if ((digitalRead(pinResetButton) == 0) || (time == 1200))
    {
    digitalWrite(audiokanal04, LOW); // Audio Ausgabe RESET
    running = 0;
    time = 0;
    schwitzminuten = 0;
    runningindicator = 0;
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "rESt" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01010000); // digit r
    Wire.write(B01111001); // digit E
    Wire.write(B01101101); // digit S
    Wire.write(B01111000); // digit t
    Wire.endTransmission();
 
    // LCD Display ansteuern
    unsigned char zeile_1[]="Saunatimer: AUS "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="                "; // Text für Zeile 2 (1)
    unsigned char zeile_3[]="Saunagang       "; // Text für Zeile 3 (2)
    unsigned char zeile_4[]="zurueck gesetzt "; // Text für Zeile 4 (3)
    LCDA.CLEAR();
    LCDA.DisplayString(0,0,zeile_1,16); // Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 2 (1) schreiben
    LCDA.DisplayString(2,0,zeile_3,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    delay(4000);
    digitalWrite(audiokanal04, HIGH);
    }
 
    // Musik an-/ausschalten (toggeln), wenn der Musik-Taster gedrückt wurde
    if ((digitalRead(pinMusikButton) == 0) && (musikflag == 0))
    {
    musikflag = 1;
    digitalWrite(audiokanal01, LOW);
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "AUon" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01110111); // digit A
    Wire.write(B00111110); // digit U
    Wire.write(B01011100); // digit o
    Wire.write(B01010100); // digit n
    Wire.endTransmission();
 
    // LCD Display ansteuern
    unsigned char zeile_1[]="Sauna - Monitor "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="                "; // Text für Zeile 2 (1)
    unsigned char zeile_3[]="Musik EIN       "; // Text für Zeile 3 (2)
    unsigned char zeile_4[]="                "; // Text für Zeile 4 (3)
    LCDA.CLEAR();
    LCDA.DisplayString(0,0,zeile_1,16); // Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 2 (1) schreiben
    LCDA.DisplayString(2,0,zeile_3,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    delay(2000);
    digitalWrite(audiokanal01, HIGH);
    }
 
    if ((digitalRead(pinMusikButton) == 0) && (musikflag == 1))
    {
    musikflag = 0;
    digitalWrite(audiokanal04, LOW);
    FourDigitLedDisplay.clearDisplay(); // LED Display löschen
    Wire.beginTransmission(saa1064); // "AUof" auf der LED Anzeige ausgeben
    Wire.write(1); // instruction byte - first digit to control is 1 (right hand side)
    Wire.write(B01110111); // digit A
    Wire.write(B00111110); // digit U
    Wire.write(B01011100); // digit o
    Wire.write(B01110001); // digit f
    Wire.endTransmission();
 
    // LCD Display ansteuern
    unsigned char zeile_1[]="Sauna - Monitor "; // Text für Zeile 1 (0)
    unsigned char zeile_2[]="                "; // Text für Zeile 2 (1)
    unsigned char zeile_3[]="Musik AUS       "; // Text für Zeile 3 (2)
    unsigned char zeile_4[]="                "; // Text für Zeile 4 (3)
    LCDA.CLEAR();
    LCDA.DisplayString(0,0,zeile_1,16); // Zeile 1 (0) schreiben
    LCDA.DisplayString(1,0,zeile_2,16); // Zeile 2 (1) schreiben
    LCDA.DisplayString(2,0,zeile_3,16); // Zeile 3 (2) schreiben
    LCDA.DisplayString(3,0,zeile_4,16); // Zeile 4 (3) schreiben
 
    delay(2000);
    digitalWrite(audiokanal04, HIGH);
    }
  }
}